# Author: Keston Smith
# Library: Caribe 
# Contact: keston.smith@my.uwi.edu
#Copyright: Apache-2.0 2021
import re
from transformers import AutoTokenizer, AutoModelForSeq2SeqLM

guyanesedict={
"bai":"boy",
"budday":"friend",
"whaam":"what happened",
"how nah":"how are you",
"waam star": "what going on brother",
"whey":"where",
"wey":"where",
"gu goo":"will go",
"ga fu":"have to",
"gadu":"have to",   
"kom":"come",
"somi":"sometimes",
"vamit":"vomit",
"yu":"you",
"fuud":"food",
"hapn":"happen",
"wid":"with",
"bredda":"brother",
"breda":"brother",
"wit":"with",
"le":"let",
"tel":"tell",
"wo":"what",
"stomik":"stomach",
"ga":"got",
"du":"do",
"yor":"your",
"dis":"this",
"kain":"kind",
"di":"the",
"swet":"sweat",
"badi":"body",
"somtaim":"sometimes",
"lef":"left",
"dem":"them",
"fut":"foot",
"wok":"work",
"waan":"want",
"markit":"market",
"muuv":"move",
"meh":"me",
"Ah":"I",
"tek":"take",
"wuk":"work",
"dong":"down",
"fu":"for",
"gu":"go",
"maan":"man",
"batam":"bottom",
"den":"then",
"aal":"all",
"awii":"all of us",
"taim":"time",
"le":"let",
"tuu":"two",
"shi":"she",
"kaal":"call",
"kozn":"cousin",
"ii":"it",
"kyaa":"cannot",
"swel":"swell",
"tugeda":"together",
"pul":"pull",
"peen":"pain",
"bos":"bust",
"das":"that is",
"tuh":"to",
"wuda":"would of",
"cuda":"could of",
"bika":"because",
"rong":"around",
"bwoy":"boy",
"notn":"nothing",
"els":"else",
"piipl":"people",
"kot":"cut",
"aks":"ask",
"bacoos":"ghosts",
"balanjay":"eggplant",
"bhaigan":"eggplant",
"luk":"look",
"da":"that",
"lef lef":"leftovers",
"pickney":"children",
"manin":"morning",
"gad":"god",
"mih":"my",
"dutty":"dirty",
"duttyness":"dirtyness",
"dah":"that",
"dat":"that",
"huu":"who",
"wid":"with",
"wai":"why",
"wails":"while",
"wen":"when",
"yuu":"you",
"awi":"we",
"vamit":"vomit",
"veen":"vein",
"tie":"tai",
"tugeda":"together",
"seef":"safe",
"chrai":"try",
"tingsz":"things",
"ting":"thing",
"der":"there",
"tablit":"tablet",
"gatu":"got to",
"ron":"run",
"dong":"down",
"das":"that is",
"evri":"every",
"da":"that",
"gaff":"talk",
"gyaff":"talk",
"gaffing":"talking",
"juk":"poke",
"one one":"several",
"sarch":"search",
"pul":"pull",
"badi":"person",
"pardno":"partner",
"peen":"pain",
"kyaa":"can't",
"kya":"can't",
"mi":"my",
"muuv":"move",
"monki":"monkey",
"lang":"long",
"innoe":"don't know",
"nah":"isn't",
"kain":"kind",
"saaf":"soft",
"mek":"make",
"yuh":"you",
"plenti":"plenty",
"gyal":"girl",
"bcuz":"because",
"tmr":"tomorrow",
  
    
}

class gec_translator:
    def __init__(self, sentence:str):
        self.sentence =sentence
        self.translation = self
        tokenizer = AutoTokenizer.from_pretrained("KES/GEC-English")
        model = AutoModelForSeq2SeqLM.from_pretrained("KES/GEC-English")
        
        inputs = tokenizer("guy:"+self.sentence, truncation=True, return_tensors='pt')
        output = model.generate(inputs['input_ids'], num_beams=4, max_length=512, early_stopping=True)
        self.translation=tokenizer.batch_decode(output, skip_special_tokens=True)
        
    
    def gec_translate(self):
        return "".join(self.translation)
    
    

class guyanese_file_encode:
    _data = []
    _out = []
    def __init__(self, filename:str, encode_filetype:str):
        self.filename = filename
        self.filetype = encode_filetype
        __mode = "r+"
        self.file = open(self.filename, __mode, encoding='utf-8') 

        for _ in self.file:
            self._data.append(_)
        for __ in self._data:
            encode = guyanese_encode(__)
            self._out.append(encode)
        self.file.close()
    
        if (self.filetype == "text" or self.filetype == "txt"):
            if (self.filename.endswith(".txt")):
                self.output = open("Translated.txt", "w")
                for x in self._out:
                    self.output.write(x)
                self.output.close()
            else:
                raise TypeError("filename type  and encode filetype mismatch")

        elif (self.filetype == "json"):
            if (self.filename.endswith(".json")):
                self.output = open("Translated.json", "w")
                for x in self._out:
                    self.output.write(x)
                self.output.close()
            else:
                raise TypeError("filename type  and encode filetype mismatch")

        elif(self.filetype == "csv"): 
            if (self.filename.endswith(".csv")):
                self.output = open("Translated.csv", "w")
                for x in self._out:
                    self.output.write(x)
                self.output.close()
            else:
                raise TypeError("filename type  and encode filetype mismatch")   

        else: 
            raise TypeError("filetype options only include 'text', 'json' or 'csv'")


def guyanese_encode(sentence:str):
    sentence=sentence.lower()
    for words in guyanesedict:
        if guyanesedict[words].lower() in sentence:
            sentence = re.sub(r'\b%s\b' %guyanesedict[words].lower(), words.lower(), sentence)
    return sentence

def guyanese_decode(sentence:str):
    sentence=sentence.lower()
    sentence = " " + sentence 
    for x in guyanesedict:
        if sentence.find(x):
            sentence = re.sub(r'\b%s\b' %x, guyanesedict[x], sentence)
    return sentence.strip()




#Author: Keston Smith

from transformers import AutoTokenizer
from datasets import load_dataset
from transformers import AutoModelForSeq2SeqLM, Seq2SeqTrainingArguments, Seq2SeqTrainer, DataCollatorForSeq2Seq

tokenizer = AutoTokenizer.from_pretrained("t5-base")
class T5_Trainer:

  def __init__(self, train_dataset_path:str, eval_dataset_path:str):
      self.model_name = "t5-base"
      self.train_dataset_path = train_dataset_path
      self.eval_dataset_path = eval_dataset_path
        
      if (not open(self.train_dataset_path) or  not open(self.eval_dataset_path)):
          raise FileNotFoundError
      
  def connect_datasets(self, format):
      self.format=format
      self.dataset = load_dataset(self.format, data_files={"train":self.train_dataset_path}, delimiter=",")
      self.dataset1 = load_dataset(self.format, data_files={"test":self.eval_dataset_path}, delimiter=",")

  def caribe_preprocessor(self, examples):
      self.examples=examples
      model_inputs = tokenizer(self.examples["input"], max_length=512, truncation=True)
      with tokenizer.as_target_tokenizer():   
          labels = tokenizer(self.examples["target"], max_length=512, truncation=True)
      model_inputs["labels"] = labels["input_ids"]
      return model_inputs


  def caribe_training(self, output_path:str, epochs:int, eval_strategy:str, l_rate, train_batch_size:int, eval_batch_size:int, decay:float, checkpoints:int):
      self.model = AutoModelForSeq2SeqLM.from_pretrained("t5-base")
      self.data_collator = DataCollatorForSeq2Seq(tokenizer=tokenizer, model=self.model)
      
      caribe_training_args = Seq2SeqTrainingArguments(
      output_dir=output_path,
      evaluation_strategy=eval_strategy,
      learning_rate=l_rate,
      per_device_train_batch_size=train_batch_size,
      per_device_eval_batch_size=eval_batch_size,
      weight_decay=decay,
      save_total_limit=checkpoints,
      num_train_epochs=epochs,)
      
      trainer = Seq2SeqTrainer(
      model=self.model,
      args=caribe_training_args,
      train_dataset=self.dataset.map(self.caribe_preprocessor, batched=True, remove_columns=["input", "target"],)['train'],
      eval_dataset=self.dataset1.map(self.caribe_preprocessor, batched=True, remove_columns=["input", "target"],)['test'],
      tokenizer=tokenizer,
      data_collator=self.data_collator,
      )
      print("\nThank you for choosing Caribe \n")
      trainer.train()
      trainer.save_model("model/")
      



#Author: Keston Smith

from transformers import AutoTokenizer, AutoModelForSeq2SeqLM
tokenizer = AutoTokenizer.from_pretrained("t5-base")
model = AutoModelForSeq2SeqLM.from_pretrained("model/", local_files_only=True)

class generate_text:
    def __init__(self, input, min_length:int= 10, max_length:int= 50, do_sample:bool = False, early_stopping:bool= False, num_beams:int= 4,temperature:float= 0.7, top_k: int = 50, no_repeat_ngram_size:int= 0, top_p: float = 1,  repetition_penalty:float=1.0):
        self.min_length=min_length
        self.max_length=max_length
        self.do_sample=do_sample
        self.early_stopping=early_stopping
        self.num_beams=num_beams
        self.temperature=temperature
        self.top_k=top_k
        self.no_repeat_ngram_size=no_repeat_ngram_size
        self.top_p=top_p
        self.repetition_penalty= repetition_penalty 
        self.input=input

    
    def output(self):    
        self.input_ids = tokenizer(self.input, return_tensors="pt").input_ids
        outputs = model.generate(self.input_ids, min_length=self.min_length, max_length=self.max_length, do_sample=self.do_sample, early_stopping=self.early_stopping, num_beams=self.num_beams, temperature=self.temperature,  top_k=self.top_k, no_repeat_ngram_size=self.no_repeat_ngram_size, top_p=self.top_p,  repetition_penalty=self.repetition_penalty)
        output_text= tokenizer.batch_decode(outputs, skip_special_tokens=True)
        return "".join(output_text)  
        



from setuptools import setup, find_packages


classifiers = [
  'Development Status :: 5 - Production/Stable',
  'Intended Audience :: Education',
  'Operating System :: Microsoft :: Windows :: Windows 10',
  'License :: OSI Approved :: Apache Software License',
  'Programming Language :: Python :: 3'
]
 
setup(
  name='Caribe',
  version='0.8.9',
  description='Caribbean English Creoles to Standard English',
  long_description_content_type="text/markdown",
  long_description=open('README.md').read() + '\n\n' + open('CHANGELOG.txt').read(),
  url='',  
  author='Keston Smith',
  author_email='keston.smith@my.uwi.edu',
  license='Apache License, Version 2.0', 
  classifiers=classifiers,
  keywords='Translator', 
  packages=find_packages(),
  install_requires=['nltk>=3.6.3', 'pandas>=1.3.4', 'gingerit==0.9.0', 'transformers>=4.15.0', 'requests>=2.25.1', 'torch', 'datasets'] 
)